/* Wallpapoz Copyright (C) 2004 Akbar <tuxer@myrealbox.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "wallpapoz.h"
#include <libxml++/libxml++.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/fileselection.h>
#include <cstdio>
#include <cstdlib>
#include <fstream>
#include <gdk/gdk.h>
using namespace std;

Glib::RefPtr<Gtk::TreeSelection> m_refTreeSelection;

Wall::Wall(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade) 
: Gtk::Window(cobject),
  m_refGlade(refGlade),
  m_pButton_about(0),
  m_pButton_quit(0),
  m_pButton_change(0),
  m_pButton_save(0),
  m_pButton_kill(0),
  m_pButton_restart(0),
  m_Picture(0),
  m_TreeView(0)
{

    pro = new Process();
    m_refGlade->get_widget("quit_button", m_pButton_quit);
    if(m_pButton_quit) {
        m_pButton_quit->signal_clicked().connect( sigc::mem_fun(*this, &Wall::on_button_quit) );
    }

    m_refGlade->get_widget("about_button", m_pButton_about);
    if(m_pButton_about) {
        m_pButton_about->signal_clicked().connect( sigc::mem_fun(*this, &Wall::on_button_about) );
    }


    m_refGlade->get_widget("treeview1", m_TreeView);
    if(m_TreeView) {
        on_tree_start();
        m_TreeView->append_column("Workspace", m_Columns.m_col_no);
        m_TreeView->append_column("Wallpaper", m_Columns.m_col_wallp);
        m_refTreeSelection = m_TreeView->get_selection();
        m_refTreeSelection->set_mode(Gtk::SELECTION_SINGLE);
        m_refTreeSelection->signal_changed().connect( sigc::mem_fun(*this, &Wall::on_selection_changed) );
    }

    m_refGlade->get_widget("change_button", m_pButton_change);
    if(m_pButton_change) {
        m_pButton_change->signal_clicked().connect( sigc::mem_fun(*this, &Wall::on_button_change) );
    }

    m_refGlade->get_widget("save_button", m_pButton_save);
    if(m_pButton_save) {
        m_pButton_save->signal_clicked().connect( sigc::mem_fun(*this, &Wall::on_button_save) );
    }

    m_refGlade->get_widget("kill_button", m_pButton_kill);
    if(m_pButton_kill) {
        m_pButton_kill->signal_clicked().connect( sigc::mem_fun(*this, &Wall::on_button_kill) );
    }

    m_refGlade->get_widget("restart_button", m_pButton_restart);
    if(m_pButton_restart) {
        m_pButton_restart->signal_clicked().connect( sigc::mem_fun(*this, &Wall::on_button_restart) );
    }

}

Wall::~Wall() {
    delete pro;
    delete m_pButton_about;
    delete m_pButton_quit;
    delete m_pButton_change;
    delete m_TreeView;

}

void Wall::on_tree_start() {
        unsigned int i = 0;
        xmlpp::DomParser parser;
        m_refTreeModel = Gtk::ListStore::create(m_Columns);
        m_TreeView->set_model(m_refTreeModel);
        xmlpp::Node* pNode;
        xmlpp::Node::NodeList list;
        xmlpp::Element *el;
        xmlpp::TextNode *t;
        Gtk::TreeModel::Row row;
        char *server_name = getenv("HOME");
        Glib::ustring filepath = server_name;
        filepath += "/.wallpapoz/wallpapoz.xml";
        ifstream in(filepath.c_str(), ios::in);

        if(in.fail()) {
            unsigned int total_desktop = pro->x_Prop();
            total_desktop -= '0';
            string action;
            action = server_name;
            action += "/.wallpapoz";
            action = "rm -rf " + action; 
            pro->spawn(action, 1);
            action = server_name;
            action += "/.wallpapoz";
            action = "mkdir -p " + action;
            pro->spawn(action, 1);
            action = server_name;
            action += "/.wallpapoz/wallpapoz.xml";
            action = "touch " + action;
            pro->spawn(action, 1);
            Glib::ustring temp2 = pro->child_Output("gconftool-2 -g /desktop/gnome/background/picture_filename");
            for(unsigned int k=0; k<total_desktop; k++) {
                row = *(m_refTreeModel->append());
                row[m_Columns.m_col_no] = k+1;
                row[m_Columns.m_col_wallp] = temp2;
            }
            on_button_save();
        }
        else {
            in.close();
            parser.parse_file(filepath);
            if(parser) {
                pNode = parser.get_document()->get_root_node();
                list = pNode->get_children();
                unsigned int space = (int)pro->x_Prop();
                space -= '0';
                xmlpp::Node::NodeList::iterator iter;
                if(space>list.size()) {
                    for(iter = list.begin(); iter != list.end(); iter++) {
                        el = dynamic_cast<xmlpp::Element*>(*iter);
                        if(!el)
                            continue;
                        t = el->get_child_text();
                        if(!t)
                            continue;
                        row = *(m_refTreeModel->append());
                        row[m_Columns.m_col_no] = ++i;
                        row[m_Columns.m_col_wallp] = t->get_content();
                    }
                    for(unsigned int j=0; j<space-list.size(); j++) {
                        row = *(m_refTreeModel->append());
                        row[m_Columns.m_col_no] = ++i;
                        row[m_Columns.m_col_wallp] = "No Background";
                    }
                    on_button_save();
                }
                else if(space<list.size()) {
                    for(iter = list.begin(); iter != list.end(); iter++) {
                        if(i==space) break;
                        el = dynamic_cast<xmlpp::Element*>(*iter);
                        if(!el)
                            continue;
                        t = el->get_child_text();
                        if(!t)
                            continue;
                        row = *(m_refTreeModel->append());
                        row[m_Columns.m_col_no] = ++i;
                        row[m_Columns.m_col_wallp] = t->get_content();
                    }
                    on_button_save();
                }
                else {
                    for(iter = list.begin(); iter != list.end(); iter++) {
                        el = dynamic_cast<xmlpp::Element*>(*iter);
                        if(!el)
                            continue;
                        t = el->get_child_text();
                        if(!t)
                            continue;
                        row = *(m_refTreeModel->append());
                        row[m_Columns.m_col_no] = ++i;
                        row[m_Columns.m_col_wallp] = t->get_content();
                    }
                }
            }
        }
}


void Wall::on_button_change() {
    Gtk::TreeModel::iterator iter = m_refTreeSelection->get_selected();
    if(iter) {
        Gtk::FileSelection dialog("Please choose a wallpaper!");
        dialog.set_transient_for(*this);
        Gtk::TreeModel::Row row2 = *iter;
        int result = dialog.run(); 
        if(result==Gtk::RESPONSE_OK) 
            row2[m_Columns.m_col_wallp] = dialog.get_filename();
    }
    on_selection_changed();
}

void Wall::on_selection_changed() {
    Gtk::TreeModel::iterator iter = m_refTreeSelection->get_selected();
    m_refGlade->get_widget("picture", m_Picture);
    if(m_Picture)
    if(iter) {
        Gtk::TreeModel::Row row2 = *iter;
        Glib::ustring temp = row2[m_Columns.m_col_wallp];
        Glib::RefPtr<Gdk::Pixbuf> m_refImage;
        m_refImage = Gdk::Pixbuf::create_from_file(temp);
        m_refImage = m_refImage->scale_simple( 610, 605, Gdk::INTERP_NEAREST );
        m_Picture->set(m_refImage);
    }
}

void Wall::on_button_kill() {
    pro->spawn("killall daemon_wallpapoz", false);
}

void Wall::on_button_restart() {
    pro->spawn("killall daemon_wallpapoz", true);
    pro->spawn("daemon_wallpapoz", false);
}

void Wall::on_button_quit() {
    hide();
}

void Wall::on_button_about() {
    Gtk::MessageDialog dialog(*this, "Wallpapoz\n Gnome Multi-Backgrounds and Wallpapers Configuration Tool\n created by Akbar");
    dialog.run();
}

void Wall::on_button_save() {
    char *server_name = getenv("HOME");
    Glib::ustring filepath = server_name;
    filepath += "/.wallpapoz/wallpapoz.xml";
    xmlpp::Document document;
    document.set_internal_subset("wallpaper", "", "");
    xmlpp::Element* nodeRoot = document.create_root_node("wallpaper");
    Gtk::TreeModel::Children children = m_refTreeModel->children();
    for(Gtk::TreeModel::Children::iterator iter=children.begin(); iter!=children.end(); iter++) {
        Gtk::TreeModel::Row row= *iter;
        Glib::ustring temp2 = row[m_Columns.m_col_wallp];
        xmlpp::Element* nodeChild = nodeRoot->add_child("file");
        nodeChild->set_child_text(temp2);
    }
    document.write_to_file(filepath);
}
